@@ -1,6 +1,6 @@
 /* A view which displays a (time, value) curve plot.
  *
- * Copyright (c) 2003 Ole Laursen.
+ * Copyright (c) 2003, 04 Ole Laursen.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -39,20 +39,18 @@ public:
   CurveView();
   ~CurveView();
   
+  static int const pixels_per_sample;
+
 private:
   virtual void do_update();
-  virtual void do_attach(Monitor *monitor, const Glib::ustring &mon_dir);
+  virtual void do_attach(Monitor *monitor);
   virtual void do_detach(Monitor *monitor);
   virtual void do_draw_loop();
 
-  unsigned int samples;		// number of samples to store
-
   // must be destroyed before the canvas
   typedef std::list<Curve *> curve_sequence;
   typedef curve_sequence::iterator curve_iterator;
   curve_sequence curves;
-
-  int draws_since_update;
 };
 
 #endif
