@@ -1,6 +1,6 @@
 /* The various system statistics - adapters of the libgtop interface.
  *
- * Copyright (c) 2003, 2004 Ole Laursen.
+ * Copyright (c) 2003, 04 Ole Laursen.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -19,7 +19,6 @@
  */
 
 #include <string>
-#include <sstream>
 #include <iomanip>
 #if __GNUC__ < 3
 #include <iostream>
@@ -41,9 +40,6 @@
 #include <glibtop/fsusage.h>
 #include <glibtop/netload.h>
 
-#if HAVE_SYS_STATFS_H
-#include <sys/statfs.h>
-#endif
 
 #include "monitor-impls.hpp"
 #include "ucompose.hpp"
@@ -55,7 +51,139 @@
 double const max_decay = 0.999; 
 
 
+//
+// functions from monitor.hpp
+//
+
+std::list<Monitor *>
+load_monitors(const Glib::RefPtr<Gnome::Conf::Client> &client,
+	      const Glib::ustring &dir)
+{
+  std::list<Monitor *> monitors;
+  
+  Glib::SListHandle<Glib::ustring> gconf_list
+    = client->all_dirs(dir + "/monitors");
+
+  for (Glib::SListHandle<Glib::ustring>::const_iterator i = gconf_list.begin(),
+	 end = gconf_list.end(); i != end; ++i) {
+    const Glib::ustring &mon_dir = *i;
+    
+    Glib::ustring type = client->get_string(mon_dir + "/type");
+    
+    if (type == "cpu_usage") {
+      Gnome::Conf::Value v = client->get(mon_dir + "/cpu_no");
+      int cpu_no;
+      
+      if (v.get_type() == Gnome::Conf::VALUE_INT)
+	cpu_no = v.get_int();
+      else
+	cpu_no = -1;
+
+      if (cpu_no == -1)
+	monitors.push_back(new CpuUsageMonitor);
+      else
+	monitors.push_back(new CpuUsageMonitor(cpu_no));
+    }
+    else if (type == "memory_usage")
+      monitors.push_back(new MemoryUsageMonitor);
+    else if (type == "swap_usage")
+      monitors.push_back(new SwapUsageMonitor);
+    else if (type == "load_average")
+      monitors.push_back(new LoadAverageMonitor);
+    else if (type == "disk_usage") {
+      Gnome::Conf::Value v = client->get(mon_dir + "/mount_dir");
+      Glib::ustring mount_dir;
+      
+      if (v.get_type() == Gnome::Conf::VALUE_STRING)
+	mount_dir = v.get_string();
+      else
+	mount_dir = "/";	// FIXME: use schema?
+
+      v = client->get(mon_dir + "/show_free");
+      bool show_free;
+
+      if (v.get_type() == Gnome::Conf::VALUE_BOOL)
+	show_free = v.get_bool();
+      else
+	show_free = false;	// FIXME: use schema?
+      
+      monitors.push_back(new DiskUsageMonitor(mount_dir, show_free));
+    }
+    else if (type == "network_load") {
+      Glib::ustring inter;
+      int inter_no, inter_direction;
+      
+      Gnome::Conf::Value v = client->get(mon_dir + "/interface");
+      
+      if (v.get_type() == Gnome::Conf::VALUE_STRING)
+	inter = v.get_string();
+      else
+	inter = "eth";	// FIXME: use schema?
+
+      
+      v = client->get(mon_dir + "/interface_no");
+      
+      if (v.get_type() == Gnome::Conf::VALUE_INT)
+	inter_no = v.get_int();
+      else
+	inter_no = 0;	// FIXME: use schema?
+
+      
+      v = client->get(mon_dir + "/interface_direction");
+      
+      if (v.get_type() == Gnome::Conf::VALUE_INT)
+	inter_direction = v.get_int();
+      else
+	inter_direction = NetworkLoadMonitor::all_data; // FIXME: use schema?
+
+      NetworkLoadMonitor::Direction dir;
+      
+      if (inter_direction == NetworkLoadMonitor::incoming_data)
+	dir = NetworkLoadMonitor::incoming_data;
+      else if (inter_direction == NetworkLoadMonitor::outgoing_data)
+	dir = NetworkLoadMonitor::outgoing_data;
+      else
+	dir = NetworkLoadMonitor::all_data;
+
+      monitors.push_back(new NetworkLoadMonitor(inter, inter_no, dir));
+    }
+    else if (type == "temperature") {
+      Gnome::Conf::Value v = client->get(mon_dir + "/temperature_no");
+      int temperature_no;
+      
+      if (v.get_type() == Gnome::Conf::VALUE_INT)
+	temperature_no = v.get_int();
+      else
+	temperature_no = 0;	// FIXME: use schema?
+      
+      monitors.push_back(new TemperatureMonitor(temperature_no));
+    }
+    else if (type == "fan_speed") {
+      Gnome::Conf::Value v = client->get(mon_dir + "/fan_no");
+      int fan_no;
+      
+      if (v.get_type() == Gnome::Conf::VALUE_INT)
+	fan_no = v.get_int();
+      else
+	fan_no = 0;	// FIXME: use schema?
+      
+      monitors.push_back(new FanSpeedMonitor(fan_no));
+    }
+
+    monitors.back()->set_gconf_dir(mon_dir);
+  }
+
+  // always start with a CpuUsageMonitor - FIXME: use schema?
+  if (monitors.empty())
+    monitors.push_back(new CpuUsageMonitor);
+
+  return monitors;
+}
+
+
+//
 // helpers
+//
 
 // for setting precision
 struct Precision
@@ -63,7 +191,8 @@ struct Precision
   int n;
 };
 
-std::ostream &operator <<(std::ostream& os, const Precision &p)
+template <typename T>
+T &operator <<(T& os, const Precision &p)
 {
 #if __GNUC__ < 3
   os << std::setprecision(p.n) << std::setiosflags(std::ios::fixed);
@@ -104,6 +233,8 @@ Precision decimal_digits(double val, int n)
 // class CpuUsageMonitor
 //
 
+int const CpuUsageMonitor::max_cpu_no = GLIBTOP_NCPU;
+
 CpuUsageMonitor::CpuUsageMonitor()
   : cpu_no(all_cpus), user_time(0), nice_time(0), sys_time(0), idle_time(0)
 {}
@@ -186,9 +317,14 @@ Glib::ustring CpuUsageMonitor::get_short_name()
     return String::ucompose(_("CPU %1"), cpu_no + 1);
 }
 
-void CpuUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			   const Glib::ustring &dir)
+int CpuUsageMonitor::update_interval()
 {
+  return 1000;
+}
+
+void CpuUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("cpu_usage"));
   client->set(dir + "/cpu_no", cpu_no);
 }
@@ -240,9 +376,14 @@ Glib::ustring SwapUsageMonitor::get_short_name()
   return _("Swap");
 }
 
-void SwapUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			    const Glib::ustring &dir)
+int SwapUsageMonitor::update_interval()
 {
+  return 10 * 1000;
+}
+
+void SwapUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("swap_usage"));
 }
 
@@ -300,10 +441,26 @@ Glib::ustring LoadAverageMonitor::get_short_name()
   return _("Load");
 }
 
-void LoadAverageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			      const Glib::ustring &dir)
+int LoadAverageMonitor::update_interval()
 {
+  return 30 * 1000;
+}
+
+void LoadAverageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("load_average"));
+  client->set(dir + "/max", max_value);
+}
+
+void LoadAverageMonitor::load(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
+  if (client->get_string(dir + "/type") == "load_average") {
+    Gnome::Conf::Value v = client->get(dir + "/max");
+    if (v.get_type() == Gnome::Conf::VALUE_FLOAT)
+      max_value = v.get_float();
+  }
 }
 
 
@@ -353,9 +510,14 @@ Glib::ustring MemoryUsageMonitor::get_short_name()
   return _("Mem.");
 }
 
-void MemoryUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			      const Glib::ustring &dir)
+int MemoryUsageMonitor::update_interval()
 {
+  return 10 * 1000;
+}
+
+void MemoryUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("memory_usage"));
 }
 
@@ -367,13 +529,6 @@ void MemoryUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
 DiskUsageMonitor::DiskUsageMonitor(const std::string &dir, bool free)
   : max_value(0), mount_dir(dir), show_free(free)
 {
-  disk_block_size = 512;	// 512 is apparently pretty common
-  
-#if HAVE_SYS_STATFS_H		// with statfs we can, however, do better
-  struct statfs buf;
-  if (statfs(mount_dir.c_str(), &buf) == 0)
-    disk_block_size = buf.f_bsize;
-#endif
 }
 
 double DiskUsageMonitor::do_measure()
@@ -386,13 +541,13 @@ double DiskUsageMonitor::do_measure()
   
   if (show_free) {
     if (fsusage.bavail > 0)
-      return fsusage.bavail;
+      return fsusage.bavail * fsusage.block_size;
     else
       return 0;
   }
   else {
     if (fsusage.blocks > 0)
-      return fsusage.blocks - fsusage.bfree;
+      return (fsusage.blocks - fsusage.bfree) * fsusage.block_size;
     else
       return 0;
   }
@@ -405,9 +560,6 @@ double DiskUsageMonitor::max()
 
 Glib::ustring DiskUsageMonitor::format_value(double val)
 {
-  // 1 block = disk_block_size bytes
-  val *= disk_block_size;
-
   if (val >= 1000 * 1000 * 1000) {
     val /= 1000 * 1000 * 1000;
     return String::ucompose(_("%1 Gb"), decimal_digits(val, 3), val);
@@ -435,9 +587,14 @@ Glib::ustring DiskUsageMonitor::get_short_name()
   return String::ucompose("%1", mount_dir);
 }
 
-void DiskUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			    const Glib::ustring &dir)
+int DiskUsageMonitor::update_interval()
 {
+  return 60 * 1000;
+}
+
+void DiskUsageMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("disk_usage"));
   client->set(dir + "/mount_dir", mount_dir);
   client->set(dir + "/show_free", show_free);
@@ -477,11 +634,21 @@ double NetworkLoadMonitor::do_measure()
 
   byte_count = bytes;
 
-  max_value = guint64(max_value * max_decay); // reduce gradually
+  if (val != 0)			// reduce gradually
+    max_value = guint64(max_value * max_decay);
   
   if (val > max_value)
     max_value = guint64(val * 1.05);
 
+  for (nlm_seq::iterator i = sync_monitors.begin(), end = sync_monitors.end();
+       i != end; ++i) {
+    NetworkLoadMonitor &other = **i;
+    if (other.max_value > max_value)
+      max_value = other.max_value;
+    else if (max_value > other.max_value)
+      other.max_value = max_value;
+  }
+  
   // calculate difference in msecs
   struct timeval tv;
   if (gettimeofday(&tv, 0) == 0) {
@@ -584,26 +751,51 @@ Glib::ustring NetworkLoadMonitor::get_short_name()
   return str;
 }
 
-void NetworkLoadMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			      const Glib::ustring &dir)
+int NetworkLoadMonitor::update_interval()
 {
+  return 1000;
+}
+
+void NetworkLoadMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("network_load"));
   client->set(dir + "/interface", interface);
   client->set(dir + "/interface_no", interface_no);
   client->set(dir + "/interface_direction", int(direction));
+  client->set(dir + "/max", int(max_value)); // hopefully 32 bits is enough
 }
 
-void NetworkLoadMonitor::possibly_sync(NetworkLoadMonitor &other)
+void NetworkLoadMonitor::load(const Glib::RefPtr<Gnome::Conf::Client> &client)
 {
-  if (interface == other.interface && interface_no == other.interface_no
-      && direction != other.direction) {
-    if (other.max_value > max_value)
-      max_value = other.max_value;
-    else if (max_value > other.max_value)
-      other.max_value = max_value;
+  Glib::ustring dir = get_gconf_dir();
+  if (client->get_string(dir + "/type") == "network_load"
+      && client->get_string(dir + "/interface") == interface
+      && client->get_int(dir + "/interface_no") == interface_no
+      && client->get_int(dir + "/interface_direction") == int(direction)) {
+    Gnome::Conf::Value v = client->get(dir + "/max");
+    if (v.get_type() == Gnome::Conf::VALUE_INT)
+      max_value = v.get_int();
   }
 }
 
+void NetworkLoadMonitor::possibly_add_sync_with(Monitor *other)
+{
+  if (NetworkLoadMonitor *o = dynamic_cast<NetworkLoadMonitor *>(other))
+    if (interface == o->interface && interface_no == o->interface_no
+	&& direction != o->direction)
+      sync_monitors.push_back(o);
+}
+
+void NetworkLoadMonitor::remove_sync_with(Monitor *other)
+{
+  nlm_seq::iterator i
+    = std::find(sync_monitors.begin(), sync_monitors.end(), other);
+
+  if (i != sync_monitors.end())
+    sync_monitors.erase(i);
+}
+
 
 //
 // implementation of sensors wrapper
@@ -781,13 +973,31 @@ Glib::ustring TemperatureMonitor::get_short_name()
   return String::ucompose(_("Temp. %1"), sensors_no + 1);
 }
 
-void TemperatureMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			      const Glib::ustring &dir)
+int TemperatureMonitor::update_interval()
 {
+  return 20 * 1000;
+}
+
+void TemperatureMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("temperature"));
   client->set(dir + "/temperature_no", sensors_no);
+  client->set(dir + "/max", max_value);
 }
 
+void TemperatureMonitor::load(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
+  if (client->get_string(dir + "/type") == "temperature"
+      && client->get_int(dir + "/temperature_no") == sensors_no) {
+    Gnome::Conf::Value v = client->get(dir + "/max");
+    if (v.get_type() == Gnome::Conf::VALUE_FLOAT)
+      max_value = v.get_float();
+  }
+}
+
+
 
 //
 // class FanSpeedMonitor
@@ -844,9 +1054,26 @@ Glib::ustring FanSpeedMonitor::get_short_name()
   return String::ucompose(_("Fan %1"), sensors_no + 1);
 }
 
-void FanSpeedMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client,
-			      const Glib::ustring &dir)
+int FanSpeedMonitor::update_interval()
 {
+  return 20 * 1000;
+}
+
+void FanSpeedMonitor::save(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
   client->set(dir + "/type", Glib::ustring("fan_speed"));
   client->set(dir + "/fan_no", sensors_no);
+  client->set(dir + "/max", max_value);
+}
+
+void FanSpeedMonitor::load(const Glib::RefPtr<Gnome::Conf::Client> &client)
+{
+  Glib::ustring dir = get_gconf_dir();
+  if (client->get_string(dir + "/type") == "fan_speed"
+      && client->get_int(dir + "/fan_no") == sensors_no) {
+    Gnome::Conf::Value v = client->get(dir + "/max");
+    if (v.get_type() == Gnome::Conf::VALUE_FLOAT)
+      max_value = v.get_float();
+  }
 }
