@@ -1,6 +1,6 @@
 /* Implementation of the PreferencesWindow class.
  *
- * Copyright (c) 2003 Ole Laursen.
+ * Copyright (c) 2003, 04 Ole Laursen.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,6 +25,8 @@
 #include <gtkmm/button.h>
 #include <gconfmm/client.h>
 
+#include <cassert>
+
 #include "preferences-window.hpp"
 #include "choose-monitor-window.hpp"
 #include "gui-helpers.hpp"
@@ -34,7 +36,7 @@
 
 
 
-PreferencesWindow::PreferencesWindow(Applet &applet_)
+PreferencesWindow::PreferencesWindow(Applet &applet_, monitor_seq monitors)
   : applet(applet_)
 {
   ui = get_glade_xml("preferences_window");
@@ -44,127 +46,79 @@ PreferencesWindow::PreferencesWindow(Applet &applet_)
   window->set_icon(applet.get_icon());
   
 
-  // connect the General tab widgets
-  ui->get_widget("update_interval_spinbutton", update_interval_spinbutton);
-  update_interval_spinbutton->signal_value_changed()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_update_interval_spinbutton_changed));
+  // connect the Viewer tab widgets
+  ui->get_widget("curve_radiobutton", curve_radiobutton);
+  curve_radiobutton->signal_toggled()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
+			on_curve_radiobutton_toggled));
+  
+  ui->get_widget("bar_radiobutton", bar_radiobutton);
+  bar_radiobutton->signal_toggled()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
+			on_bar_radiobutton_toggled));
+  
+  ui->get_widget("column_radiobutton", column_radiobutton);
+  column_radiobutton->signal_toggled()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
+			on_column_radiobutton_toggled));
+  
+  ui->get_widget("text_radiobutton", text_radiobutton);
+  text_radiobutton->signal_toggled()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
+			on_text_radiobutton_toggled));
+
+  ui->get_widget("flame_radiobutton", flame_radiobutton);
+  flame_radiobutton->signal_toggled()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
+			on_flame_radiobutton_toggled));
+  
+  ui->get_widget("size_outer_vbox", size_outer_vbox);
+
+  ui->get_widget("size_scale", size_scale);
+  size_scale_cb =  size_scale->signal_value_changed()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_size_scale_changed));
+
+  ui->get_widget("font_outer_vbox", font_outer_vbox);
+
+  ui->get_widget("font_checkbutton", font_checkbutton);
+  font_checkbutton->signal_toggled()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
+			on_font_checkbutton_toggled));
+
+  ui->get_widget("fontpicker", fontpicker);
+  fontpicker->signal_font_set()
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_fontpicker_set));
+
 
   ui->get_widget("background_colorpicker", background_colorpicker);
   background_colorpicker->signal_color_set()
-    .connect(SigC::slot(*this, &PreferencesWindow::
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
 			on_background_colorpicker_set));
 
   ui->get_widget("panel_background_radiobutton", panel_background_radiobutton);
   ui->get_widget("background_color_radiobutton", background_color_radiobutton);
   background_color_radiobutton->signal_toggled()
-    .connect(SigC::slot(*this, &PreferencesWindow::
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::
 			on_background_color_radiobutton_toggled));
   
   
-  // connect the Viewer tab radiobuttons
-  ui->get_widget("curve_radiobutton", curve_radiobutton);
-  curve_radiobutton->signal_toggled()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_curve_radiobutton_toggled));
-  
-  ui->get_widget("bar_radiobutton", bar_radiobutton);
-  bar_radiobutton->signal_toggled()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_bar_radiobutton_toggled));
-  
-  ui->get_widget("column_radiobutton", column_radiobutton);
-  column_radiobutton->signal_toggled()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_column_radiobutton_toggled));
-  
-  ui->get_widget("text_radiobutton", text_radiobutton);
-  text_radiobutton->signal_toggled()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_text_radiobutton_toggled));
-
-  ui->get_widget("flame_radiobutton", flame_radiobutton);
-  flame_radiobutton->signal_toggled()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_flame_radiobutton_toggled));
-  
-  ui->get_widget("curve_options", curve_options);
-
-  ui->get_widget("curve_size_spinbutton", curve_size_spinbutton);
-  curve_size_spinbutton->signal_value_changed()
-    .connect(SigC::bind(SigC::slot(*this, &PreferencesWindow::
-				   on_size_spinbutton_changed),
-			curve_size_spinbutton));
-
-  ui->get_widget("curve_samples_spinbutton", curve_samples_spinbutton);
-  curve_samples_spinbutton->signal_value_changed()
-    .connect(SigC::bind(SigC::slot(*this, &PreferencesWindow::
-				   on_samples_spinbutton_changed),
-			curve_samples_spinbutton));
-
-
-  ui->get_widget("bar_options", bar_options);
-
-  ui->get_widget("bar_size_spinbutton", bar_size_spinbutton);
-  bar_size_spinbutton->signal_value_changed()
-    .connect(SigC::bind(SigC::slot(*this, &PreferencesWindow::
-				   on_size_spinbutton_changed),
-			bar_size_spinbutton));
-
-
-  ui->get_widget("column_options", column_options);
-
-  ui->get_widget("column_size_spinbutton", column_size_spinbutton);
-  column_size_spinbutton->signal_value_changed()
-    .connect(SigC::bind(SigC::slot(*this, &PreferencesWindow::
-				   on_size_spinbutton_changed),
-			column_size_spinbutton));
-
-  ui->get_widget("column_samples_spinbutton", column_samples_spinbutton);
-  column_samples_spinbutton->signal_value_changed()
-    .connect(SigC::bind(SigC::slot(*this, &PreferencesWindow::
-				   on_samples_spinbutton_changed),
-			column_samples_spinbutton));
-
-
-  ui->get_widget("text_options", text_options);
-
-  ui->get_widget("font_checkbutton", font_checkbutton);
-  font_checkbutton->signal_toggled()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_font_checkbutton_toggled));
-
-  ui->get_widget("fontpicker", fontpicker);
-  fontpicker->signal_font_set()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_fontpicker_set));
-
-
-  ui->get_widget("flame_options", flame_options);
-
-  ui->get_widget("flame_size_spinbutton", flame_size_spinbutton);
-  flame_size_spinbutton->signal_value_changed()
-    .connect(SigC::bind(SigC::slot(*this, &PreferencesWindow::
-				   on_size_spinbutton_changed),
-			flame_size_spinbutton));
-
-  
   // connect the Monitor tab widgets
   Gtk::Button *add_button;
   ui->get_widget("add_button", add_button);
   add_button->signal_clicked()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_add_button_clicked));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_add_button_clicked));
 
   ui->get_widget("remove_button", remove_button);
   remove_button->signal_clicked()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_remove_button_clicked));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_remove_button_clicked));
 
   ui->get_widget("change_button", change_button);
   change_button->signal_clicked()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_change_button_clicked));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_change_button_clicked));
 
   ui->get_widget("monitor_treeview", monitor_treeview);
   monitor_treeview->get_selection()->signal_changed()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_selection_changed));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_selection_changed));
 
   ui->get_widget("monitor_options", monitor_options);
 
@@ -175,28 +129,24 @@ PreferencesWindow::PreferencesWindow(Applet &applet_)
   monitor_treeview->append_column(_("Device"), mc.name);
   
   ui->get_widget("monitor_curve_options", monitor_curve_options);
-  ui->get_widget("line_width_spinbutton", line_width_spinbutton);
-  line_width_spinbutton->signal_value_changed()
-    .connect(SigC::slot(*this, &PreferencesWindow::
-			on_line_width_spinbutton_changed));
   ui->get_widget("line_colorpicker", line_colorpicker);
   line_colorpicker->signal_color_set()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_monitor_colorpicker_set));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
 
   ui->get_widget("monitor_bar_options", monitor_bar_options);
   ui->get_widget("bar_colorpicker", bar_colorpicker);
   bar_colorpicker->signal_color_set()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_monitor_colorpicker_set));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
   
   ui->get_widget("monitor_column_options", monitor_column_options);
   ui->get_widget("column_colorpicker", column_colorpicker);
   column_colorpicker->signal_color_set()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_monitor_colorpicker_set));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
 
   ui->get_widget("monitor_flame_options", monitor_flame_options);
   ui->get_widget("flame_colorpicker", flame_colorpicker);
   flame_colorpicker->signal_color_set()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_monitor_colorpicker_set));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_monitor_colorpicker_set));
   
   
   // connect GConf
@@ -204,48 +154,33 @@ PreferencesWindow::PreferencesWindow(Applet &applet_)
   Glib::ustring dir = applet.get_gconf_dir();
   
   client->notify_add(dir + "/viewer_type",
-		     SigC::slot(*this, &PreferencesWindow::
+		     sigc::mem_fun(*this, &PreferencesWindow::
 				viewer_type_listener));
 
-  client->notify_add(dir + "/update_interval",
-		     SigC::slot(*this, &PreferencesWindow::
-				update_interval_listener));
-
   client->notify_add(dir + "/background_interval",
-		     SigC::slot(*this, &PreferencesWindow::
+		     sigc::mem_fun(*this, &PreferencesWindow::
 				background_color_listener));
 
   client->notify_add(dir + "/viewer/size",
-		     SigC::slot(*this, &PreferencesWindow::size_listener));
-
-  client->notify_add(dir + "/viewer/samples",
-		     SigC::slot(*this, &PreferencesWindow::samples_listener));
+		     sigc::mem_fun(*this, &PreferencesWindow::size_listener));
 
   client->notify_add(dir + "/viewer/font",
-		     SigC::slot(*this, &PreferencesWindow::font_listener));
+		     sigc::mem_fun(*this, &PreferencesWindow::font_listener));
 
-  client->notify_add(dir + "/monitors",
-		     SigC::slot(*this, &PreferencesWindow::monitors_listener));
-  
   
   // fill in values
   viewer_type_listener(0, client->get_entry(dir + "/viewer_type"));
-  update_interval_listener(0, client->get_entry(dir + "/update_interval"));
   background_color_listener(0, client->get_entry(dir + "/background_color"));
   use_background_color_listener(0, client
 				->get_entry(dir + "/use_background_color"));
   size_listener(0, client->get_entry(dir + "/viewer/size"));
-  samples_listener(0, client->get_entry(dir + "/viewer/samples"));
   font_listener(0, client->get_entry(dir + "/viewer/font"));
 
-  Glib::SListHandle<Glib::ustring> list
-    = applet.get_gconf_client()->all_dirs(applet.get_gconf_dir() + "/monitors");
-  for (Glib::SListHandle<Glib::ustring>::const_iterator i = list.begin(),
-	 end = list.end(); i != end; ++i)
-    monitors_listener(0, client->get_entry(*i + "/type"));
-
-  // this allows the user to discover the relationship between the grayed-out
-  // buttons and the treeview
+  for (monitor_iter i = monitors.begin(), end = monitors.end(); i != end; ++i)
+    add_to_monitors_list(*i);
+  
+  // deselect all to allow the user to discover the relationship
+  // between the greyed-out buttons and the treeview
   monitor_treeview->get_selection()->unselect_all();
 
   // make sure background colorpicker is grayed out
@@ -256,10 +191,10 @@ PreferencesWindow::PreferencesWindow(Applet &applet_)
   ui->get_widget("close_button", close_button);
 
   close_button->signal_clicked()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_close_button_clicked));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_close_button_clicked));
 
   window->signal_delete_event()
-    .connect(SigC::slot(*this, &PreferencesWindow::on_closed));
+    .connect(sigc::mem_fun(*this, &PreferencesWindow::on_closed));
 }
 
 PreferencesWindow::~PreferencesWindow()
@@ -288,46 +223,34 @@ void PreferencesWindow::viewer_type_listener(unsigned int,
   if (s == "curve") {
     if (!curve_radiobutton->get_active())
       curve_radiobutton->property_active() = true;
-    curve_options->property_visible() = true;
+    size_outer_vbox->property_visible() = true;
     monitor_curve_options->property_visible() = true;
   }
   else if (s == "bar") {
     if (!bar_radiobutton->get_active())
       bar_radiobutton->property_active() = true;
-    bar_options->property_visible() = true;
+    size_outer_vbox->property_visible() = true;
     monitor_bar_options->property_visible() = true;
   }
   else if (s == "column") {
     if (!column_radiobutton->get_active())
       column_radiobutton->property_active() = true;
-    column_options->property_visible() = true;
+    size_outer_vbox->property_visible() = true;
     monitor_column_options->property_visible() = true;
   }
   else if (s == "text") {
     if (!text_radiobutton->get_active())
       text_radiobutton->property_active() = true;
-    text_options->property_visible() = true;
+    font_outer_vbox->property_visible() = true;
   }
   else if (s == "flame") {
     if (!flame_radiobutton->get_active())
       flame_radiobutton->property_active() = true;
-    flame_options->property_visible() = true;
+    size_outer_vbox->property_visible() = true;
     monitor_flame_options->property_visible() = true;
   }
 }
 
-void PreferencesWindow::update_interval_listener(unsigned int,
-						 Gnome::Conf::Entry entry)
-{
-  if (entry.get_value().get_type() != Gnome::Conf::VALUE_FLOAT)
-    return;
-
-  double d = entry.get_value().get_float();
-  
-  if (update_interval_spinbutton->get_value() != d)
-    update_interval_spinbutton->set_value(d);
-}
-
 void PreferencesWindow::background_color_listener(unsigned int,
 						  Gnome::Conf::Entry entry)
 {
@@ -366,32 +289,8 @@ void PreferencesWindow::size_listener(unsigned int,
 
   int i = entry.get_value().get_int();
   
-  if (curve_size_spinbutton->get_value() != i)
-    curve_size_spinbutton->set_value(i);
-
-  if (bar_size_spinbutton->get_value() != i)
-    bar_size_spinbutton->set_value(i);
-
-  if (column_size_spinbutton->get_value() != i)
-    column_size_spinbutton->set_value(i);
-
-  if (flame_size_spinbutton->get_value() != i)
-    flame_size_spinbutton->set_value(i);
-}
-
-void PreferencesWindow::samples_listener(unsigned int,
-					 Gnome::Conf::Entry entry)
-{
-  if (entry.get_value().get_type() != Gnome::Conf::VALUE_INT)
-    return;
-
-  int i = entry.get_value().get_int();
-  
-  if (curve_samples_spinbutton->get_value() != i)
-    curve_samples_spinbutton->set_value(i);
-
-  if (column_samples_spinbutton->get_value() != i)
-    column_samples_spinbutton->set_value(i);
+  if (size_scale_to_pixels(int(size_scale->get_value())) != i)
+    size_scale->set_value(pixels_to_size_scale(i));
 }
 
 void PreferencesWindow::font_listener(unsigned int, Gnome::Conf::Entry entry)
@@ -410,53 +309,6 @@ void PreferencesWindow::font_listener(unsigned int, Gnome::Conf::Entry entry)
   }
 }
 
-void PreferencesWindow::monitors_listener(unsigned int,
-					  Gnome::Conf::Entry entry)
-{
-  static MonitorColumns mc;
-
-  Glib::ustring key = entry.get_key();
-  if (key.substr(key.size() - 5) != "/type")
-    return;
-
-  Glib::ustring mdir = key.substr(0, key.size() - 5);
-
-  store_iter
-    pos = monitor_store->children().begin(),
-    end = monitor_store->children().end();
-  while (pos != end && (*pos)[mc.dir] != mdir)
-    ++pos;
-
-  bool exists = pos != monitor_store->children().end();
-
-  // value was added
-  if (entry.get_value().get_type() == Gnome::Conf::VALUE_STRING) {
-    if (!exists) {
-      // add value
-      store_iter i = monitor_store->append();
-      (*i)[mc.name] = applet.look_up_monitor(mdir)->get_name();
-      (*i)[mc.dir] = mdir;
-      
-      monitor_treeview->get_selection()->select(i);
-    }
-  }
-  else				// value was removed
-    if (exists)
-      monitor_store->erase(pos);
-}
-
-void PreferencesWindow::line_width_listener(unsigned int,
-					    Gnome::Conf::Entry entry)
-{
-  if (entry.get_value().get_type() != Gnome::Conf::VALUE_FLOAT)
-    return;
-
-  double d = entry.get_value().get_float();
-  
-  if (line_width_spinbutton->get_value() != d)
-    line_width_spinbutton->set_value(d);
-}
-
 void PreferencesWindow::monitor_color_listener(unsigned int,
 					       Gnome::Conf::Entry entry)
 {
@@ -490,15 +342,6 @@ void PreferencesWindow::monitor_color_listener(unsigned int,
 
 // UI callbacks
 
-void PreferencesWindow::on_update_interval_spinbutton_changed()
-{
-  Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
-  Glib::ustring dir = applet.get_gconf_dir();
-  
-  double d = update_interval_spinbutton->get_value();
-  client->set(dir + "/update_interval", d);
-}
-
 unsigned int to_int(unsigned int r, unsigned int g, unsigned int b,
 		    unsigned int a)
 {
@@ -539,7 +382,7 @@ void PreferencesWindow::on_curve_radiobutton_toggled()
   if (active)
     client->set(dir + "/viewer_type", Glib::ustring("curve"));
 
-  curve_options->property_visible() = active;
+  size_outer_vbox->property_visible() = active;
   monitor_curve_options->property_visible() = active;
 }
 
@@ -553,7 +396,7 @@ void PreferencesWindow::on_bar_radiobutton_toggled()
   if (active)
     client->set(dir + "/viewer_type", Glib::ustring("bar"));
 
-  bar_options->property_visible() = active;
+  size_outer_vbox->property_visible() = active;
   monitor_bar_options->property_visible() = active;
 }
 
@@ -567,7 +410,7 @@ void PreferencesWindow::on_column_radiobutton_toggled()
   if (active)
     client->set(dir + "/viewer_type", Glib::ustring("column"));
 
-  column_options->property_visible() = active;
+  size_outer_vbox->property_visible() = active;
   monitor_column_options->property_visible() = active;
 }
 
@@ -581,7 +424,7 @@ void PreferencesWindow::on_text_radiobutton_toggled()
   if (active)
     client->set(dir + "/viewer_type", Glib::ustring("text"));
 
-  text_options->property_visible() = active;
+  font_outer_vbox->property_visible() = active;
 }
 
 void PreferencesWindow::on_flame_radiobutton_toggled()
@@ -594,26 +437,20 @@ void PreferencesWindow::on_flame_radiobutton_toggled()
   if (active)
     client->set(dir + "/viewer_type", Glib::ustring("flame"));
 
-  flame_options->property_visible() = active;
+  size_outer_vbox->property_visible() = active;
   monitor_flame_options->property_visible() = active;
 }
 
-void PreferencesWindow::on_size_spinbutton_changed(Gtk::SpinButton *size_sb)
+void PreferencesWindow::on_size_scale_changed()
 {
   Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
   Glib::ustring dir = applet.get_gconf_dir();
 
-  int i = int(size_sb->get_value());
-  client->set(dir + "/viewer/size", i);
-}
-
-void PreferencesWindow::on_samples_spinbutton_changed(Gtk::SpinButton *samples_sb)
-{
-  Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
-  Glib::ustring dir = applet.get_gconf_dir();
-
-  int i = int(samples_sb->get_value());
-  client->set(dir + "/viewer/samples", i);
+  size_scale_cb.block();
+  int i = int(size_scale->get_value() + 0.5);
+  size_scale->set_value(i);
+  client->set(dir + "/viewer/size", size_scale_to_pixels(i));
+  size_scale_cb.unblock();
 }
 
 void PreferencesWindow::on_font_checkbutton_toggled()
@@ -641,13 +478,12 @@ void PreferencesWindow::on_fontpicker_set(const Glib::ustring &font)
 
 void PreferencesWindow::on_add_button_clicked()
 {
-  ChooseMonitorWindow window;
-  window.set_icon(applet.get_icon());
+  Monitor *monitor = run_choose_monitor_window(Glib::ustring());
 
-  Monitor *monitor = window.run(applet.get_gconf_client(), Glib::ustring());
-
-  if (monitor)
+  if (monitor) {
     applet.add_monitor(monitor);
+    add_to_monitors_list(monitor);
+  }
 }
 
 void PreferencesWindow::on_remove_button_clicked()
@@ -656,8 +492,11 @@ void PreferencesWindow::on_remove_button_clicked()
   
   store_iter i = monitor_treeview->get_selection()->get_selected();
   
-  if (i)
-    applet.remove_monitor((*i)[mc.dir]);
+  if (i) {
+    Monitor *mon = (*i)[mc.monitor];
+    monitor_store->erase(i);
+    applet.remove_monitor(mon);
+  }
 }
 
 void PreferencesWindow::on_change_button_clicked()
@@ -667,17 +506,15 @@ void PreferencesWindow::on_change_button_clicked()
   store_iter i = monitor_treeview->get_selection()->get_selected();
   
   if (i) {
-    Glib::ustring mon_dir = (*i)[mc.dir];
-    
-    ChooseMonitorWindow window;
-    window.set_icon(applet.get_icon());
+    Monitor *prev_monitor = (*i)[mc.monitor];
+    Monitor *new_monitor
+      = run_choose_monitor_window(prev_monitor->get_gconf_dir());
 
-    Monitor *monitor = window.run(applet.get_gconf_client(), mon_dir);
+    if (new_monitor) {
+      applet.replace_monitor(prev_monitor, new_monitor);
 
-    if (monitor) {
-      applet.replace_monitor(mon_dir, monitor);
-
-      (*i)[mc.name] = monitor->get_name();
+      (*i)[mc.name] = new_monitor->get_name();
+      (*i)[mc.monitor] = new_monitor;
     }
   }
 }
@@ -705,19 +542,11 @@ void PreferencesWindow::on_selection_changed()
   stop_monitor_listeners();
 
   if (sel) {
-    Glib::ustring key;
+    Glib::ustring key, mon_dir = (*(*i)[mc.monitor]).get_gconf_dir();
     unsigned int con;
 
-    key = (*i)[mc.dir] + "/line_width";
-    con = client->notify_add(key, SigC::slot(*this, &PreferencesWindow::
-					     line_width_listener));
-
-    line_width_listener(0, client->get_entry(key));
-
-    monitor_listeners.push_back(con);
-
-    key = (*i)[mc.dir] + "/color";
-    con = client->notify_add(key, SigC::slot(*this, &PreferencesWindow::
+    key = mon_dir + "/color";
+    con = client->notify_add(key, sigc::mem_fun(*this, &PreferencesWindow::
 					     monitor_color_listener));
 
     monitor_color_listener(0, client->get_entry(key));
@@ -730,22 +559,6 @@ void PreferencesWindow::on_selection_changed()
   monitor_options->set_sensitive(sel);
 }
 
-void PreferencesWindow::on_line_width_spinbutton_changed()
-{
-  static MonitorColumns mc;
-  
-  Glib::RefPtr<Gnome::Conf::Client> &client = applet.get_gconf_client();
-
-  store_iter i = monitor_treeview->get_selection()->get_selected();
-  
-  if (i) {
-    Glib::ustring mon_dir = (*i)[mc.dir];
-
-    double d = line_width_spinbutton->get_value();
-    client->set(mon_dir + "/line_width", d);
-  }
-}
-
 void PreferencesWindow::on_monitor_colorpicker_set(unsigned int r,
 						   unsigned int g,
 						   unsigned int b,
@@ -758,7 +571,7 @@ void PreferencesWindow::on_monitor_colorpicker_set(unsigned int r,
   store_iter i = monitor_treeview->get_selection()->get_selected();
   
   if (i) {
-    Glib::ustring mon_dir = (*i)[mc.dir];
+    Glib::ustring mon_dir = (*(*i)[mc.monitor]).get_gconf_dir();
     client->set(mon_dir + "/color", int(to_int(r / 256, g / 256,
 					       b / 256, a / 256)));
   }
@@ -774,3 +587,51 @@ bool PreferencesWindow::on_closed(GdkEventAny *)
   window->hide();
   return false;
 }
+
+Monitor *PreferencesWindow::run_choose_monitor_window(const Glib::ustring &str)
+{
+  ChooseMonitorWindow window(applet.get_icon(), *window);
+  
+  return window.run(applet.get_gconf_client(), str);
+}
+
+void PreferencesWindow::add_to_monitors_list(Monitor *mon)
+{
+  MonitorColumns mc;
+  
+  store_iter i = monitor_store->append();
+  (*i)[mc.name] = mon->get_name();
+  (*i)[mc.monitor] = mon;
+      
+  monitor_treeview->get_selection()->select(i);
+}
+
+
+// for converting between size_scale units and pixels
+int const pixel_size_table_size = 10;
+int pixel_size_table[pixel_size_table_size]
+  = { 32, 48, 64, 96, 128, 192, 256, 384, 512, 1024 };
+
+
+int PreferencesWindow::size_scale_to_pixels(int size)
+{
+  assert(size >= 0 && size < pixel_size_table_size);
+
+  return pixel_size_table[size];
+}
+
+int PreferencesWindow::pixels_to_size_scale(int pixels)
+{
+  // we may not have an exact match, so just find the nearest
+  int min_diff, min_i;
+  for (int i = 0; i < pixel_size_table_size; ++i) {
+    int diff = std::abs(pixel_size_table[i] - pixels);
+    if (diff < min_diff) {
+      min_diff = diff;
+      min_i = i;
+    }
+  }
+
+  return min_i;
+}
+
