@@ -1,6 +1,6 @@
 /* A textual monitoring view.
  *
- * Copyright (c) 2003 Ole Laursen.
+ * Copyright (c) 2003, 04 Ole Laursen.
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -32,10 +32,13 @@ class Text;
 
 class TextView: public View
 {
+public:
+  TextView();
+  
 private:
   virtual void do_display();
   virtual void do_update();
-  virtual void do_attach(Monitor *monitor, const Glib::ustring &mon_dir);
+  virtual void do_attach(Monitor *monitor);
   virtual void do_detach(Monitor *monitor);
   virtual void do_set_background(unsigned int color);
   virtual void do_unset_background();
